﻿using Grpc.Core;
using Grpc.DotNet.SharedLib.Generated;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace Grpc.DotNet.ClientApp
{
    internal class CallbackHandler
    {
        readonly IAsyncStreamReader<BidirectionalCalculatorReply> _responseStream;
        readonly CancellationToken _cancellationToken;

        public CallbackHandler(IAsyncStreamReader<BidirectionalCalculatorReply> responseStream)
            : this(responseStream, CancellationToken.None) { }

        public CallbackHandler(IAsyncStreamReader<BidirectionalCalculatorReply> responseStream,
            CancellationToken cancellationToken)
        {
            _responseStream = responseStream ?? throw new ArgumentNullException(nameof(responseStream));
            _cancellationToken = cancellationToken;
            Task = Task.Run(Consume, _cancellationToken);
        }

        public Task Task { get; }

        async Task Consume()
        {
            while (await _responseStream.MoveNext(_cancellationToken).ConfigureAwait(false))
                Console.WriteLine("Equals({0}), Equation({1})", _responseStream.Current.Result, _responseStream.Current.Eqn);
        }
    }
}
