﻿using Grpc.Net.Client;
using Grpc.DotNet.SharedLib.Generated;
using System;
using System.Threading.Tasks;
using Grpc.Core;

namespace Grpc.DotNet.ClientApp
{
    class Program
    {
        static async Task Main(string[] args)
        {
            using (GrpcChannel channel = GrpcChannel.ForAddress("https://localhost:5001"))
            {
                try
                {
                    CalculatorService.CalculatorServiceClient client =
                        new CalculatorService.CalculatorServiceClient(channel);

                    const double N1 = 1;
                    const double N2 = 1;
                    CalculatorReply result =
                        await client.AddAsync(new CalculatorRequest() { N1 = N1, N2 = N2 });
                    Console.WriteLine($"{N1} + {N2} = {result.Result}");

                    BidirectionalCalculatorService.BidirectionalCalculatorServiceClient bidirectionalClient =
                        new BidirectionalCalculatorService.BidirectionalCalculatorServiceClient(channel);

                    using (var duplexStream = bidirectionalClient.Calculate())
                    {
                        var callbackHandler = new CallbackHandler(duplexStream.ResponseStream);

                        await duplexStream.RequestStream.WriteAsync(new BidirectionalCalculatorRequest()
                        {
                            Operation = Operation.Add,
                            N = 2
                        });

                        await duplexStream.RequestStream.WriteAsync(new BidirectionalCalculatorRequest()
                        {
                            Operation = Operation.Multiply,
                            N = 2
                        });

                        await duplexStream.RequestStream.WriteAsync(new BidirectionalCalculatorRequest()
                        {
                            Operation = Operation.Subtract,
                            N = 2
                        });

                        await duplexStream.RequestStream.WriteAsync(new BidirectionalCalculatorRequest()
                        {
                            Operation = Operation.Clear
                        });

                        await duplexStream.RequestStream.WriteAsync(new BidirectionalCalculatorRequest()
                        {
                            Operation = Operation.Add,
                            N = 10
                        });

                        await duplexStream.RequestStream.CompleteAsync();
                        await callbackHandler.Task;
                    }
                }
                catch (RpcException ex)
                {
                    Console.WriteLine(ex.Status.Detail);
                }
            }
        }
    }
}

